@@ -1,8 +1,9 @@
 from __future__ import annotations
 from typing import Dict, List, Optional, Any
+from time import sleep
 from libtmux import Server, Session, Window, Pane
 from libtmux.exc import LibTmuxException
-from dotenv import dotenv_values
+from rich.console import Console
 import os
 import sys
 import json
@@ -11,6 +12,8 @@ import pickle
 
 from parasut_cli.config.settings import APP_DIR, env
 
+console = Console()
+
 
 class Receiver:
     def __init__(self) -> None:
@@ -60,7 +63,6 @@ class Receiver:
         except KeyError as e:
             print(f"Please set environment variable: {e}")
             sys.exit(0)
-
         self._tmux_server: Server
         self._tmux_session_parasut_ws_setup: Session
         self._tmux_session_parasut_ws_editor: Session
@@ -72,11 +74,20 @@ class Receiver:
             ui_library="ui-library",
             shared_logic="shared-logic",
         )
+        self._core_commands: Dict[str, str] = dict(
+            source_nvm="source ~/.nvm/nvm.sh",
+            source_yvm="source ~/.yvm/yvm.sh",
+            source_rvm="source ~/.rvm/scripts/rvm",
+        )
         self._server_commands: Dict[str, str] = dict(
             launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
             choose_ruby_version=f"rvm use {self.SERVER_RUBY_V}",
             launch_sidekiq="bundle exec sidekiq",
             launch_rails="rails server",
+            switch_to_phoenix=f"rails runner 'puts Company.find({self.COMPANY_ID}).update!(used_app: \"{self.PHOENIX_SWITCH_NAME}\")'",
+            switch_to_trinity=f"rails runner 'puts Company.find({self.COMPANY_ID}).update!(used_app: \"{self.TRINITY_SWITCH_NAME}\")'",
+            switch_to_receipt=f"rails runner 'puts company=Company.find({self.COMPANY_ID}); company.feature_flags[\"using_sales_receipt\"]=true; company.save!'",
+            switch_to_invoice=f"rails runner 'puts company=Company.find({self.COMPANY_ID}); company.feature_flags[\"using_sales_receipt\"]=false; company.save!'",
         )
         self._billing_commands: Dict[str, str] = dict(
             launch_text_editor=self.PARASUT_CLI_TEXT_EDITOR,
@@ -120,10 +131,160 @@ class Receiver:
             choose_node_version=f"nvm use {self.SHARED_LOGIC_NODE_V}",
             ember_serve=f"ember s --live-reload-port {self.SHARED_LOGIC_EMBER_PORT}",
         )
+        # run tasks
+        self._task_run_server = " && ".join(
+            [
+                self._core_commands["source_rvm"],
+                self._server_commands["choose_ruby_version"],
+                self._server_commands["launch_rails"],
+            ]
+        )
+        self._task_run_server_sidekiq = " && ".join(
+            [
+                self._core_commands["source_rvm"],
+                self._server_commands["choose_ruby_version"],
+                self._server_commands["launch_sidekiq"],
+            ]
+        )
+        self._task_run_billing = " && ".join(
+            [
+                self._core_commands["source_rvm"],
+                self._billing_commands["choose_ruby_version"],
+                self._billing_commands["launch_rails"],
+            ]
+        )
+        self._task_run_billing_sidekiq = " && ".join(
+            [
+                self._core_commands["source_rvm"],
+                self._billing_commands["choose_ruby_version"],
+                self._billing_commands["launch_sidekiq"],
+            ]
+        )
+        self._task_run_e_doc_broker = " && ".join(
+            [
+                self._core_commands["source_rvm"],
+                self._e_doc_broker_commands["choose_ruby_version"],
+                self._e_doc_broker_commands["launch_rails"],
+            ]
+        )
+        self._task_run_e_doc_broker_sidekiq = " && ".join(
+            [
+                self._core_commands["source_rvm"],
+                self._e_doc_broker_commands["choose_ruby_version"],
+                self._e_doc_broker_commands["launch_sidekiq"],
+            ]
+        )
+        self._task_run_phoenix = " && ".join(
+            [
+                self._core_commands["source_yvm"],
+                self._core_commands["source_nvm"],
+                self._phoenix_commands["choose_yarn_version"],
+                self._phoenix_commands["choose_node_version"],
+                self._phoenix_commands["ember_serve"],
+            ]
+        )
+        self._task_run_shared_logic = " && ".join(
+            [
+                self._core_commands["source_yvm"],
+                self._core_commands["source_nvm"],
+                self._shared_logic_commands["choose_yarn_version"],
+                self._shared_logic_commands["choose_node_version"],
+                self._shared_logic_commands["ember_serve"],
+            ]
+        )
+        self._task_run_trinity = " && ".join(
+            [
+                self._core_commands["source_yvm"],
+                self._core_commands["source_nvm"],
+                self._trinity_commands["choose_yarn_version"],
+                self._trinity_commands["choose_node_version"],
+                self._trinity_commands["ember_serve"],
+            ]
+        )
+        self._task_run_ui_library = " && ".join(
+            [
+                self._core_commands["source_yvm"],
+                self._core_commands["source_nvm"],
+                self._ui_library_commands["choose_yarn_version"],
+                self._ui_library_commands["choose_node_version"],
+                self._ui_library_commands["ember_serve"],
+            ]
+        )
+        self._task_run_client = " && ".join(
+            [
+                self._core_commands["source_yvm"],
+                self._core_commands["source_nvm"],
+                self._client_commands["choose_yarn_version"],
+                self._client_commands["choose_node_version"],
+                self._client_commands["ember_serve"],
+            ]
+        )
+        # switch tasks
+        self._task_switch_frontend_to_phoenix = " && ".join(
+            [
+                self._core_commands["source_rvm"],
+                self._server_commands["choose_ruby_version"],
+                self._server_commands["switch_to_phoenix"],
+            ]
+        )
+        self._task_switch_frontend_to_trinity = " && ".join(
+            [
+                self._core_commands["source_rvm"],
+                self._server_commands["choose_ruby_version"],
+                self._server_commands["switch_to_trinity"],
+            ]
+        )
+        self._task_switch_addling_to_invoice = " && ".join(
+            [
+                self._core_commands["source_rvm"],
+                self._server_commands["choose_ruby_version"],
+                self._server_commands["switch_to_invoice"],
+            ]
+        )
+        self._task_switch_addling_to_receipt = " && ".join(
+            [
+                self._core_commands["source_rvm"],
+                self._server_commands["choose_ruby_version"],
+                self._server_commands["switch_to_receipt"],
+            ]
+        )
 
     def initialize_tmux_server(self) -> None:
         self._tmux_server = Server()
 
+    def run_repo(self, repo_name: str) -> None:
+        base_path: str = self._find_repo_path(repo_name)
+
+        self._change_directory(base_path)
+
+        try:
+            if "server" == repo_name:
+                self._run_process([self._task_run_server])
+            elif "server-sidekiq" == repo_name:
+                self._run_process([self._task_run_server_sidekiq])
+            elif "billing" == repo_name:
+                self._run_process([self._task_run_billing])
+            elif "billing-sidekiq" == repo_name:
+                self._run_process([self._task_run_billing_sidekiq])
+            elif "e-doc-broker" == repo_name:
+                self._run_process([self._task_run_e_doc_broker])
+            elif "e-doc-broker-sidekiq" == repo_name:
+                self._run_process([self._task_run_e_doc_broker_sidekiq])
+            elif "phoenix" == repo_name:
+                self._run_process([self._task_run_phoenix])
+            elif "shared-logic" == repo_name:
+                self._run_process([self._task_run_shared_logic])
+            elif "trinity" == repo_name:
+                self._run_process([self._task_run_trinity])
+            elif "ui-library" == repo_name:
+                self._run_process([self._task_run_ui_library])
+            elif "client" == repo_name:
+                self._run_process([self._task_run_client])
+        except KeyboardInterrupt:
+            pass
+        except Exception as e:
+            print(e)
+
     def create_parasut_ws_setup(self, repos: List[str]) -> None:
         session: Optional[Session]
 
@@ -146,9 +307,7 @@ class Receiver:
                 self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.SERVER_DIR}")
                 self._launch_parasut_server_repo()
             elif "billing" == repo_name:
-                self._change_directory(
-                    f"{self.PARASUT_BASE_DIR}/{self.BILLING_DIR}"
-                )
+                self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.BILLING_DIR}")
                 self._launch_parasut_billing_repo()
             elif "e-doc-broker" == repo_name:
                 self._change_directory(
@@ -156,22 +315,16 @@ class Receiver:
                 )
                 self._launch_parasut_e_doc_broker_repo()
             elif "phoenix" == repo_name:
-                self._change_directory(
-                    f"{self.PARASUT_BASE_DIR}/{self.PHOENIX_DIR}"
-                )
+                self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.PHOENIX_DIR}")
                 self._launch_parasut_phoenix_repo()
             elif "client" == repo_name:
                 self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.CLIENT_DIR}")
                 self._launch_parasut_client_repo()
             elif "trinity" == repo_name:
-                self._change_directory(
-                    f"{self.PARASUT_BASE_DIR}/{self.TRINITY_DIR}"
-                )
+                self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.TRINITY_DIR}")
                 self._launch_parasut_trinity_repo()
             elif "ui-library" == repo_name:
-                self._change_directory(
-                    f"{self.PARASUT_BASE_DIR}/{self.UI_LIBRARY_DIR}"
-                )
+                self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.UI_LIBRARY_DIR}")
                 self._launch_parasut_ui_library_repo()
             elif "shared-logic" == repo_name:
                 self._change_directory(
@@ -188,7 +341,9 @@ class Receiver:
 
         # get or create session
         try:
-            session = self._tmux_server.find_where({"session_name": "parasut-ws-editor"})
+            session = self._tmux_server.find_where(
+                {"session_name": "parasut-ws-editor"}
+            )
         except LibTmuxException:
             session = None
         finally:
@@ -205,9 +360,7 @@ class Receiver:
                 self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.SERVER_DIR}")
                 self._launch_parasut_server_editor()
             elif "billing" == repo_name:
-                self._change_directory(
-                    f"{self.PARASUT_BASE_DIR}/{self.BILLING_DIR}"
-                )
+                self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.BILLING_DIR}")
                 self._launch_parasut_billing_editor()
             elif "e-doc-broker" == repo_name:
                 self._change_directory(
@@ -215,22 +368,16 @@ class Receiver:
                 )
                 self._launch_parasut_e_doc_broker_editor()
             elif "phoenix" == repo_name:
-                self._change_directory(
-                    f"{self.PARASUT_BASE_DIR}/{self.PHOENIX_DIR}"
-                )
+                self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.PHOENIX_DIR}")
                 self._launch_parasut_phoenix_editor()
             elif "client" == repo_name:
                 self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.CLIENT_DIR}")
                 self._launch_parasut_client_editor()
             elif "trinity" == repo_name:
-                self._change_directory(
-                    f"{self.PARASUT_BASE_DIR}/{self.TRINITY_DIR}"
-                )
+                self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.TRINITY_DIR}")
                 self._launch_parasut_trinity_editor()
             elif "ui-library" == repo_name:
-                self._change_directory(
-                    f"{self.PARASUT_BASE_DIR}/{self.UI_LIBRARY_DIR}"
-                )
+                self._change_directory(f"{self.PARASUT_BASE_DIR}/{self.UI_LIBRARY_DIR}")
                 self._launch_parasut_ui_library_editor()
             elif "shared-logic" == repo_name:
                 self._change_directory(
@@ -242,55 +389,59 @@ class Receiver:
             self._tmux_session_parasut_ws_editor.select_window(1).kill_window()
         self._tmux_session_parasut_ws_editor.select_window(1)
 
-    def switch_server_rails_frontend(self, target_repo: str) -> None:
+    def switch_server_rails_frontend(self, target_repo: str, show_output: bool) -> None:
         server_repo: str = f"{self.PARASUT_BASE_DIR}/{self.SERVER_DIR}"
 
         self._change_directory(server_repo)
 
-        if target_repo == "phoenix":
-            subprocess.run(
-                [
-                    "/bin/zsh",
-                    "-i",
-                    "-c",
-                    f"rvm use {self.SERVER_RUBY_V} && rails runner 'puts Company.find({self.COMPANY_ID}).update!(used_app: \"{self.PHOENIX_SWITCH_NAME}\")'",
-                ]
-            )
-        if target_repo == "trinity":
-            subprocess.run(
-                [
-                    "/bin/zsh",
-                    "-i",
-                    "-c",
-                    f"rvm use {self.SERVER_RUBY_V} && rails runner 'puts Company.find({self.COMPANY_ID}).update!(used_app: \"{self.TRINITY_SWITCH_NAME}\")'",
-                ]
+        try:
+            if target_repo == "phoenix":
+                self._run_process(
+                    tasks=[self._task_switch_frontend_to_phoenix],
+                    show_output=show_output,
+                )
+                if show_output is not True : console.print(f":clinking_beer_mugs: Demand accomplished.")
+            if target_repo == "trinity":
+                self._run_process(
+                    tasks=[self._task_switch_frontend_to_trinity],
+                    show_output=show_output,
+                )
+                if show_output is not True : console.print(f":clinking_beer_mugs: Demand accomplished.")
+        except KeyboardInterrupt:
+            console.print(
+                f":pile_of_poo: You interrupted process. Manually check your demand steps."
             )
+            sys.exit(0)
 
-    def switch_server_rails_addling(self, target_addling: str) -> None:
+    def switch_server_rails_addling(
+        self, target_addling: str, show_output: bool
+    ) -> None:
         server_repo: str = f"{self.PARASUT_BASE_DIR}/{self.SERVER_DIR}"
 
         self._change_directory(server_repo)
 
-        if target_addling == "receipt":
-            subprocess.run(
-                [
-                    "/bin/zsh",
-                    "-i",
-                    "-c",
-                    f"rvm use {self.SERVER_RUBY_V} && rails runner 'puts company=Company.find({self.COMPANY_ID}); company.feature_flags[\"using_sales_receipt\"]=true; company.save!'",
-                ]
-            )
-        if target_addling == "invoice":
-            subprocess.run(
-                [
-                    "/bin/zsh",
-                    "-i",
-                    "-c",
-                    f"rvm use {self.SERVER_RUBY_V} && rails runner 'puts company=Company.find({self.COMPANY_ID}); company.feature_flags[\"using_sales_receipt\"]=false; company.save!'",
-                ]
+        try:
+            if target_addling == "receipt":
+                self._run_process(
+                    tasks=[self._task_switch_addling_to_receipt],
+                    show_output=show_output,
+                )
+                if show_output is not True : console.print(f":clinking_beer_mugs: Demand accomplished.")
+            if target_addling == "invoice":
+                self._run_process(
+                    tasks=[self._task_switch_addling_to_invoice],
+                    show_output=show_output,
+                )
+                if show_output is not True : console.print(f":clinking_beer_mugs: Demand accomplished.")
+        except KeyboardInterrupt:
+            console.print(
+                f":pile_of_poo: You interrupted process. Manually check your demand steps."
             )
+            sys.exit(0)
 
-    def do_linking(self, base_repo: str, target_repos: List[str]) -> None:
+    def do_linking(
+        self, base_repo: str, target_repos: List[str], show_output: bool
+    ) -> None:
         base_path: str = self._find_repo_path(base_repo)
         dep_key: str = ""
         dep_value: str = ""
@@ -307,7 +458,7 @@ class Receiver:
                 target_path = f"{self.PARASUT_BASE_DIR}/{self.UI_LIBRARY_DIR}"
                 json_file = "package.json"
 
-                if self._dep_versions["ui_library"]["linked"] == True:
+                if self._dep_versions["ui_library"]["linked"] is True:
                     print(
                         f"{repo_name} has been linked before. Try --list to check linked repos."
                     )
@@ -320,17 +471,23 @@ class Receiver:
                     self._dep_versions["ui_library"]["linked"] = True
 
                     self._store_linking_info(self._dep_versions)
-                    self._apply_package_changes()
+                    self._apply_package_changes(show_output=show_output)
+                    if show_output is not True:
+                        console.print(f":ok_hand: Base repo linked.")
                     self._change_directory(target_path)
-                    self._apply_package_changes(force=True)
+                    self._apply_package_changes(show_output=show_output, force=True)
+                    if show_output is not True:
+                        console.print(f":ok_hand: Target repo reloaded.")
                     self._change_directory(base_path)
+                    if show_output is not True:
+                        console.print(f":clinking_beer_mugs: Demand accomplished.")
             elif repo_name == "shared-logic":
                 dep_key = "shared-logic"
                 dep_value = f"link:../{repo_name}"
                 target_path = f"{self.PARASUT_BASE_DIR}/{self.SHARED_LOGIC_DIR}"
                 json_file = "package.json"
 
-                if self._dep_versions["shared_logic"]["linked"] == True:
+                if self._dep_versions["shared_logic"]["linked"] is True:
                     print(
                         f"{repo_name} has been linked before. Try --list to check linked repos."
                     )
@@ -343,12 +500,18 @@ class Receiver:
                     self._dep_versions["shared_logic"]["linked"] = True
 
                     self._store_linking_info(self._dep_versions)
-                    self._apply_package_changes()
+                    self._apply_package_changes(show_output=show_output)
+                    if show_output is not True:
+                        console.print(f":ok_hand: Base repo linked.")
                     self._change_directory(target_path)
-                    self._apply_package_changes(force=True)
+                    self._apply_package_changes(show_output=show_output, force=True)
+                    if show_output is not True:
+                        console.print(f":ok_hand: Target repo reloaded.")
                     self._change_directory(base_path)
+                    if show_output is not True:
+                        console.print(f":clinking_beer_mugs: Demand accomplished.")
 
-    def undo_linking(self, base_repo: str, repos: List[str]) -> None:
+    def undo_linking(self, base_repo: str, repos: List[str], show_output: bool) -> None:
         base_path: str = self._find_repo_path(base_repo)
         dep_key: str = ""
         dep_value: str = ""
@@ -365,7 +528,7 @@ class Receiver:
                 target_path = f"{self.PARASUT_BASE_DIR}/{self.UI_LIBRARY_DIR}"
                 json_file = "package.json"
 
-                if self._dep_versions["ui_library"]["linked"] == False:
+                if self._dep_versions["ui_library"]["linked"] is False:
                     print(
                         f"{repo_name} has not been linked before. Try listing linked repos."
                     )
@@ -375,17 +538,23 @@ class Receiver:
                     )
                     self._dep_versions["ui_library"]["linked"] = False
 
-                    self._apply_package_changes()
+                    self._apply_package_changes(show_output=show_output)
+                    if show_output is not True:
+                        console.print(f":ok_hand: Base repo unlinked.")
                     self._change_directory(target_path)
-                    self._apply_package_changes(force=True)
+                    self._apply_package_changes(show_output=show_output, force=True)
+                    if show_output is not True:
+                        console.print(f":ok_hand: Undoed target repo reloaded.")
                     self._change_directory(base_path)
+                    if show_output is not True:
+                        console.print(f":clinking_beer_mugs: Demand accomplished.")
             elif repo_name == "shared-logic":
                 dep_key = "shared-logic"
                 dep_value = self._dep_versions["shared_logic"]["value"]
                 target_path = f"{self.PARASUT_BASE_DIR}/{self.SHARED_LOGIC_DIR}"
                 json_file = "package.json"
 
-                if self._dep_versions["shared_logic"]["linked"] == False:
+                if self._dep_versions["shared_logic"]["linked"] is False:
                     print(
                         f"{repo_name} has not been linked before. Try listing linked repos."
                     )
@@ -395,33 +564,61 @@ class Receiver:
                     )
                     self._dep_versions["shared_logic"]["linked"] = False
 
-                    self._apply_package_changes()
+                    self._apply_package_changes(show_output=show_output)
+                    if show_output is not True:
+                        console.print(f":ok_hand: Base repo unlinked.")
                     self._change_directory(target_path)
-                    self._apply_package_changes(force=True)
+                    self._apply_package_changes(show_output=show_output, force=True)
+                    if show_output is not True:
+                        console.print(f":ok_hand: Undoed target repo reloaded.")
                     self._change_directory(base_path)
+                    if show_output is not True:
+                        console.print(f":clinking_beer_mugs: Demand accomplished.")
 
     def get_linked_repos(self, base_repo: str) -> None:
         for key, value in self._dep_versions.items():
             if self._dep_versions[key]["linked"]:
                 print(key)
         else:
-            print("There is no repo linked by this cli.")
+            print("There is no repo linking to this repo.")
 
-    def _apply_package_changes(self, force=False) -> None:
+    def _run_process(self, tasks: List[str], check=False, show_output=False):
+        if show_output is False:
+            with console.status("[bold green] Working on process...") as status:
+                for task in tasks:
+                    result = subprocess.run(
+                        ["/bin/zsh", "-c", f"{task}"],
+                        capture_output=True,
+                    )
+                    result.check_returncode()
+        else:
+            for task in tasks:
+                subprocess.run(["/bin/zsh", "-c", f"{task}"], check=check)
+
+    def _apply_package_changes(self, show_output: bool, force: bool = False) -> None:
         attempts = 0
+        command = f"yarn install{' --force' if force else ''}"
 
         while attempts < 3:
             try:
-                subprocess.run(
-                    ["/bin/zsh", "-c", f'yarn install{" --force" if force else ""}'],
-                    check=True,
-                )
+                self._run_process(tasks=[command], check=True, show_output=show_output)
                 break
             except subprocess.CalledProcessError:
+                if show_output is not True:
+                    console.print(
+                        f":thumbs_down: Yarn had a problem with an dependent package."
+                    )
+                    if attempts == 2:
+                        sleep(1)
+                        console.print(f":goblin: &$!#%")
+                        sleep(1)
+                        console.print(f":goblin::fire: Trying last time..")
+                    else:
+                        console.print(f":fire: Trying again..")
                 attempts += 1
         else:
-            raise Exception(
-                "Exiting because of an error: yarn couldn't install dependencies on related repo"
+            console.print(
+                f":broken_heart: Yarn couldn't install dependencies on related repo."
             )
 
     def _change_dependency_value(
@@ -438,11 +635,11 @@ class Receiver:
         return dep_ver
 
     def _find_repo_path(self, repo_name: str) -> str:
-        if repo_name == "server":
+        if repo_name == "server" or "server" in repo_name:
             return f"{self.PARASUT_BASE_DIR}/{self.SERVER_DIR}"
-        elif repo_name == "billing":
+        elif repo_name == "billing" or "billing" in repo_name:
             return f"{self.PARASUT_BASE_DIR}/{self.BILLING_DIR}"
-        elif repo_name == "e-doc-broker":
+        elif repo_name == "e-doc-broker" or "e-doc-broker" in repo_name:
             return f"{self.PARASUT_BASE_DIR}/{self.E_DOC_BROKER_DIR}"
         elif repo_name == "client":
             return f"{self.PARASUT_BASE_DIR}/{self.CLIENT_DIR}"
